
#include <ctime>
#include <string.h>

#include <ilcplex/ilocplex.h>

ILOSTLBEGIN

using namespace std;

int main(int argc, char **argv)
{
	try {
		IloEnv env;
		IloModel model(env, "Demonstration");
		IloNumVarArray x;
		x = IloNumVarArray(env, 3, 0.0, IloInfinity, ILOFLOAT);

		// dfinition de la fonction objectif
		// ----------------------------------
		IloExpr obj(env);
		IloNumArray Cost(env, 3);
		Cost[0] = 4;
		Cost[1] = 2;
		Cost[2] = 1;
		obj = IloScalProd(Cost, x);
		model.add(IloMinimize(env, obj));

		// ajout de la contrainte 1.
		// -------------------------
		IloRangeArray cover(env);

		IloRange r1(env, 1.1, x[0] + x[1], IloInfinity, "C0");
		cover.add(r1);

		// ajout de la contrainte 2.
		// -------------------------

		// creation expressions
		IloNumExpr cov(env);
		cov = x[1] + x[2];
		// ajout
		IloRange cc1(env, 3.2, cov, IloInfinity, "C1");
		cover.add(cc1);

		// ajout de la contrainte 3.
		// -------------------------

		// creation expressions
		// IloNumExpr cov(env);
		cov = x[0] + x[2];
		// ajout
		IloRange cc2(env, 4.3, cov, IloInfinity, "C2");
		cover.add(cc2);

		// ajout des contraintes au modle
		// -------------------------
		model.add(cover);

		// sauvegarde sur le disque
		// -------------------------
		IloCplex cplex(env);
		cplex.extract(model);
		cplex.exportModel("ModelDW.lp");

		// rsolution
		cplex.solve();

		float BS = (float)cplex.getObjValue();

		float valeur1 = (float)cplex.getValue(x[0]);
		float valeur2 = (float)cplex.getValue(x[1]);
		float valeur3 = (float)cplex.getValue(x[2]);

		cout << "valeur solution = " << BS << endl;
		cout << "x[1] = " << valeur1 << endl;
		cout << "x[2] = " << valeur2 << endl;
		cout << "x[3] = " << valeur3 << endl;

		float variable_duale_1 = (float)cplex.getDual(cover[0]);
		float variable_duale_2 = (float)cplex.getDual(cover[1]);
		float variable_duale_3 = (float)cplex.getDual(cover[2]);

		cout << "variable duale 1 = " << variable_duale_1 << endl;
		cout << "variable duale 2 = " << variable_duale_2 << endl;
		cout << "variable duale 3 = " << variable_duale_3 << endl;
	}
	catch (IloException& e) {
		cerr << "Concert exception caught: " << e << endl;
	}
}
